@@ -1,20 +1,16 @@
-import re
 import json
-from enum import Enum
 from pathlib import Path
-from functools import lru_cache
 from typing import List, Optional, Dict
 
+from nonebot.adapters.onebot.v11 import Message
 from nonebot.log import logger
 
-
-class MatchType(Enum):
-    congruence = 1  # 全匹配(==)
-    include = 2  # 模糊匹配(in)
-    regex = 3  # 正则匹配(regex)
+from .models import MatchType
+from .word_entry import WordEntry
+from .util import compare_msg
 
 
-NULL_BANK = {t.name: {"0": {}} for t in MatchType}
+NULL_BANK = {t.name: {"0": []} for t in MatchType}
 
 
 class WordBank(object):
@@ -24,14 +20,19 @@ class WordBank(object):
         self.img_dir = self.data_dir / "img"
         self.data_dir.mkdir(parents=True, exist_ok=True)
         self.img_dir.mkdir(parents=True, exist_ok=True)
-        self.__data: Dict[str, Dict[str, Dict[str, list]]] = {}
+        self.__data: Dict[str, Dict[str, List[WordEntry]]] = {}
         self.__load()
 
     def __load(self):
         if self.bank_path.exists() and self.bank_path.is_file():
             with self.bank_path.open("r", encoding="utf-8") as f:
-                data: dict = json.load(f)
-            self.__data = {t.name: data.get(t.name) or {"0": {}} for t in MatchType}
+                data: Dict[str, Dict[str, Dict[str, List[str]]]] = json.load(f)
+            for t in MatchType:
+                self.__data[t.name] = {}
+                for user_id in data.get(t.name, {}).keys():
+                    self.__data[t.name][user_id] = []
+                    for key, value in data[t.name][user_id].items():
+                        self.__data[t.name][user_id].append(WordEntry.load(key, value))
             logger.success("读取词库位于 " + str(self.bank_path))
         else:
             self.__data = NULL_BANK
@@ -39,18 +40,24 @@ class WordBank(object):
             logger.success("创建词库位于 " + str(self.bank_path))
 
     def __save(self):
+        data: Dict[str, Dict[str, Dict[str, List[str]]]] = {}
+        for t in self.__data.keys():
+            data[t] = {}
+            for user_id in self.__data[t].keys():
+                data[t][user_id] = {}
+                for entry in self.__data[t][user_id]:
+                    key, values = entry.dump()
+                    data[t][user_id][key] = values
         with self.bank_path.open("w", encoding="utf-8") as f:
-            json.dump(self.__data, f, ensure_ascii=False, indent=4)
-        self.match.cache_clear()
+            json.dump(data, f, ensure_ascii=False, indent=4)
 
-    @lru_cache(100)
     def match(
         self,
         index: str,
-        msg: str,
+        msg: Message,
         match_type: Optional[MatchType] = None,
         to_me: bool = False,
-    ) -> Optional[List]:
+    ) -> List[Message]:
         """
         匹配词条
 
@@ -67,40 +74,30 @@ class WordBank(object):
                 res = self.__match(index, msg, type_, to_me)
                 if res:
                     return res
+            return []
         else:
             return self.__match(index, msg, match_type, to_me)
 
     def __match(
-        self, index: str, msg: str, match_type: MatchType, to_me: bool = False
-    ) -> Optional[List]:
+        self, index: str, msg: Message, match_type: MatchType, to_me: bool = False
+    ) -> List[Message]:
 
-        bank: Dict[str, list] = dict(
-            self.__data[match_type.name].get(index, {}),
-            **self.__data[match_type.name].get("0", {}),
-        )
+        bank: List[WordEntry] = self.__data[match_type.name].get(index, [])
+        bank += self.__data[match_type.name].get("0", [])
 
-        # event.to_me 时优先匹配带"/atme "的词条
-        if match_type == MatchType.congruence:
-            return (bank.get(f"/atme {msg}", []) if to_me else []) or bank.get(msg, [])
+        for entry in bank:
+            if entry.match(msg, match_type, to_me):
+                return entry.get_values()
+        return []
 
-        elif match_type == MatchType.include:
-            for key in bank:
-                if to_me and key.startswith("/atme "):
-                    key = key[6:]
-                if key in msg:
-                    return bank[key]
-
-        elif match_type == MatchType.regex:
-            for key in bank:
-                if to_me and key.startswith("/atme "):
-                    key = key[6:]
-                try:
-                    if re.search(key, msg, re.S):
-                        return bank[key]
-                except re.error:
-                    logger.error(f"正则匹配错误 - pattern: {key}, string: {msg}")
-
-    def set(self, index: str, key: str, value: str, match_type: MatchType) -> bool:
+    def set(
+        self,
+        index: str,
+        match_type: MatchType,
+        key: Message,
+        value: Message,
+        require_to_me: bool = False,
+    ) -> bool:
         """
         新增词条
 
@@ -113,17 +110,29 @@ class WordBank(object):
         :return:
         """
         name = match_type.name
-        if self.__data[name].get(index, {}):
-            if self.__data[name][index].get(key, []):
-                self.__data[name][index][key].append(value)
-            else:
-                self.__data[name][index][key] = [value]
+        add = False
+        if index in self.__data[name]:
+            for entry in self.__data[name][index]:
+                if entry.require_to_me != require_to_me:
+                    continue
+                if compare_msg(entry.key, key):
+                    entry.add_value(value)
+                    add = True
+                    break
         else:
-            self.__data[name][index] = {key: [value]}
+            self.__data[name][index] = []
+        if not add:
+            self.__data[name][index].append(WordEntry(key, [value], require_to_me))
         self.__save()
         return True
 
-    def delete(self, index: str, key: str, match_type: MatchType) -> bool:
+    def delete(
+        self,
+        index: str,
+        match_type: MatchType,
+        key: Message,
+        require_to_me: bool = False,
+    ) -> bool:
         """
         删除词条
 
@@ -135,10 +144,13 @@ class WordBank(object):
         :return:
         """
         name = match_type.name
-        if self.__data[name].get(index, {}).get(key, False):
-            del self.__data[name][index][key]
-            self.__save()
-            return True
+        for entry in list(self.__data[name].get(index, [])):
+            if entry.require_to_me != require_to_me:
+                continue
+            if compare_msg(entry.key, key):
+                self.__data[name][index].remove(entry)
+                self.__save()
+                return True
         return False
 
     def clear(self, index: str) -> bool:
@@ -153,7 +165,7 @@ class WordBank(object):
         else:
             for type_ in MatchType:
                 name = type_.name
-                if self.__data[name].get(index, {}):
+                if index in self.__data[name]:
                     del self.__data[name][index]
         self.__save()
         return True
